import java.util.NoSuchElementException;

public class BinTree<T> {
 private Node<T> root;
 /** Erzeugt leeren Baum */
 public BinTree() {this.root = null;}
 /** Erzeugt Baum mit einem einzigen Knoten */
 public BinTree(T v){
  this.root = new Node<T>(null,v,null);     
 }
 
 /** Konstruiert Baum aus zwei gegebenen Teilbaeumen und
 einer neuen Wurzel mit Wert v */
 public BinTree(BinTree<T> left, T v, BinTree<T> right){
  this.root = new Node<T>(left.root,v,right.root);
 } 
 /** Prueft, ob der Baum leer ist */
 public boolean isEmpty() {
  return this.root == null;
 }
 
 /** Liefert den Wert der Wurzel zurueck */
 public T getRootValue() {
  if (this.root == null) {
  throw new NoSuchElementException("Ein leerer Baum hat keine Wurzel!");
  }
  else {return this.root.getValue();}
 }

 /** Liefert den Wert der Wurzel zurueck */
 public Node<T> getRoot() {
  if (this.root == null) {
  throw new NoSuchElementException("Ein leerer Baum hat keine Wurzel!");
  }
  else {return this.root;}
 }

 
 /** Liefert den linken Teilbaum unter der Wurzel */
 public BinTree<T> getLeft() {
  if (this.root == null) {
  throw new NoSuchElementException("Ein leerer Baum hat keinen linken Teilbaum");
  }
  else {
   BinTree<T> result = new BinTree<T>();
   result.root = this.root.getLeft();
   return result;}
 }
 /** Liefert den rechten Teilbaum unter der Wurzel */
 public BinTree<T> getRight() {
  if (this.root == null) {
  throw new NoSuchElementException("Ein leerer Baum hat keinen rechten Teilbaum");
  }
  else {
   BinTree<T> result = new BinTree<T>();
   result.root = this.root.getRight();
   return result;}
 } 
 /** Elegante (aber ineffiziente) Berechnung der Groesse */
 public int sizeElegant() {
  if (this.isEmpty()) {return 0;} // Basisfall
  else {return 
             1+this.getLeft().sizeElegant() + this.getRight().sizeElegant();
        }
 
}
 /** Effizientere Berechnung der Groesse */
 public int size() {
  if (this.isEmpty()) {return 0;} // Basisfall
  else {return this.root.size();}
 }
 
  /** Elegante (aber ineffiziente) Berechnung der Tiefe */
 public int depthElegant() {
  if (this.isEmpty()) {
      throw new IllegalArgumentException("Ein leerer Baum hat keine Tiefe.");
  }
  
  if (this.getLeft().isEmpty()) {
      if (this.getRight().isEmpty()) {return 0;}
      else {return 1+this.getRight().depthElegant();}
  }
  else {
      if (this.getRight().isEmpty()) {
        return 1+this.getLeft().depthElegant();
        }
      else {return 1+Math.max(this.getLeft().depthElegant(),this.getRight().depthElegant());}
 }
}
 /** Effizientere Berechnung der Tiefe */
 public int depth() {
  if (this.isEmpty()) {
      throw new IllegalArgumentException("Ein leerer Baum hat keine Tiefe.");
  }
  else {return this.root.depth();}
 }

 
 
 public DFIterator<T> dfIterator() {
  return (new DFIterator<T>(this.root));
 }
 public BFIterator<T> bfIterator() {
  return (new BFIterator<T>(this.root));
 } 
}
 
